==> shelf_bind-0.6.0.tar.gz: lib/src/binding/response_bindings.dart <==
// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_bind.bindings.response;

import 'package:shelf/shelf.dart' show Request, Response;
import 'dart:mirrors' show ClosureMirror, InstanceMirror;
import '../preconditions.dart' show ensure, isNotNull;
import '../annotations.dart' show ResponseHeaders;
import 'dart:convert' show JSON;
import 'package:path/path.dart' as p show url;
import 'dart:io' show HttpStatus;
import 'mirror_util.dart' show getAnnotation;
import 'package:shelf_exception_response/exception.dart';
import 'validation.dart' ;
import '../util.dart';

final _url = p.url;


ResponseBinding inferResponseBinding(ClosureMirror functionMirror,
      ResponseBinding providedResponseBinding, bool validateReturn) {
  if (providedResponseBinding != null) {
    return providedResponseBinding; // TODO: should probably merge
  }

  ResponseHeaders responseHeaders =
      getAnnotation(functionMirror.function, ResponseHeaders);

  return responseHeaders != null ?
      new JsonResponseBinding(responseHeaders.successStatus,
          setLocation: responseHeaders.setLocation,
          idField: responseHeaders.idField,
          validateReturn: validateReturn) :
            new JsonResponseBinding.ok(validateReturn: validateReturn);
}


abstract class ResponseBinding {
  const ResponseBinding();

  createResponse(Request request, InstanceMirror responseMirror);
  void _validate();
}

abstract class BaseResponseBinding extends ResponseBinding {
  final int successStatus;
  final bool setLocation;
  final Symbol idField;
  final bool validateReturn;

  const BaseResponseBinding(this.successStatus, {this.setLocation: false,
    this.idField: #id, this.validateReturn: false});

  const BaseResponseBinding.ok({ bool validateReturn: false })
      : this(HttpStatus.OK, validateReturn: validateReturn);

  const BaseResponseBinding.created({ bool validateReturn: false })
      : this(HttpStatus.CREATED, setLocation: true,
          validateReturn: validateReturn);

  createResponse(Request request, InstanceMirror responseMirror) {
    final r = responseMirror.reflectee;
    if (r == null) {
      return new Response.internalServerError(body:
        'handler method returned null');
    }

    if (r is Response) {
      return r;
    }

    return doCreateResponse(request, responseMirror);
  }

  Response doCreateResponse(Request request, InstanceMirror responseMirror) {
    _validateReturn(responseMirror.reflectee);
    final body = createBody(request, responseMirror);
    final statusCode = createStatusCode(request, responseMirror, body);
    final headers = createHeaders(request, responseMirror, body);
    return new Response(statusCode, body: body, headers: headers);
  }

  _validateReturn(object) {
    if (validateReturn) {
      final violations = validator.validate(object);
      if (violations.isNotEmpty) {
        throw new HttpException(HttpStatus.INTERNAL_SERVER_ERROR,
            "Internal Server Error", { 'errors': iterableToJson(violations) });
      }
    }
  }

  createBody(Request request, InstanceMirror responseMirror);

  int createStatusCode(Request request, InstanceMirror responseMirror,
                       dynamic body) => successStatus;

  Map<String, String> createHeaders(Request request,
      InstanceMirror responseMirror, dynamic body) {
    final headers = {};
    if (setLocation) {
      final location = _getLocation(request, responseMirror);
      headers['Location'] = location;
    }
    addHeaders(headers, request, responseMirror, body);
    return headers;
  }

  String _getLocation(Request request, InstanceMirror responseMirror) {
    if (request.method != 'POST') {
      return request.requestedUri.toString();
    }

    final String id = getId(request, responseMirror);
    final requestPath = _url.fromUri(request.requestedUri);
    return _url.join(requestPath, id);
  }

  String getId(Request request, InstanceMirror responseMirror) {
    final id = responseMirror.getField(idField);
    return id.reflectee;
  }

  void addHeaders(Map<String, String> headers,
                  Request request, InstanceMirror responseMirror,
                  dynamic body) {}

  @override
  void _validate() {
    ensure(successStatus, isNotNull);
  }
}

class JsonResponseBinding extends BaseResponseBinding {

  const JsonResponseBinding(int successStatus, {bool setLocation: false,
    Symbol idField: #id, bool validateReturn: false
  })
      : super(successStatus, setLocation: setLocation, idField: idField,
          validateReturn: validateReturn);

  const JsonResponseBinding.ok({ bool validateReturn: false })
      : super.ok(validateReturn: validateReturn);

  const JsonResponseBinding.created({ bool validateReturn: false })
      : super.created(validateReturn: validateReturn);

  @override
  createBody(Request request, InstanceMirror responseMirror) =>
      JSON.encode(responseMirror.reflectee);

  @override
  void addHeaders(Map<String, String> headers,
                  Request request, InstanceMirror responseMirror,
                  dynamic body) {
      headers['Content-Type'] = 'application/json';
  }

}



typedef _BoundHandler(List<dynamic> positionalArguments, Request request,
    ResponseBinding responseBinding);

==> shelf_bind-0.6.1.tar.gz: lib/src/binding/response_bindings.dart <==
// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_bind.bindings.response;

import 'package:shelf/shelf.dart' show Request, Response;
import 'dart:mirrors' show ClosureMirror, InstanceMirror;
import '../preconditions.dart' show ensure, isNotNull;
import '../annotations.dart' show ResponseHeaders;
import 'dart:convert' show JSON;
import 'package:path/path.dart' as p show url;
import 'dart:io' show HttpStatus;
import 'mirror_util.dart' show getAnnotation;

final _url = p.url;


ResponseBinding inferResponseBinding(ClosureMirror functionMirror,
      ResponseBinding providedResponseBinding) {
  if (providedResponseBinding != null) {
    return providedResponseBinding; // TODO: should probably merge
  }

  ResponseHeaders responseHeaders =
      getAnnotation(functionMirror.function, ResponseHeaders);

  return responseHeaders != null ?
      new JsonResponseBinding(responseHeaders.successStatus,
          setLocation: responseHeaders.setLocation,
          idField: responseHeaders.idField) :
            new JsonResponseBinding.ok();
}


abstract class ResponseBinding {
  const ResponseBinding();

  createResponse(Request request, InstanceMirror responseMirror);
  void _validate();
}

abstract class BaseResponseBinding extends ResponseBinding {
  final int successStatus;
  final bool setLocation;
  final Symbol idField;

  const BaseResponseBinding(this.successStatus, {this.setLocation: false,
    this.idField: #id});

  const BaseResponseBinding.ok()
      : this(HttpStatus.OK);

  const BaseResponseBinding.created()
      : this(HttpStatus.CREATED, setLocation: true);

  createResponse(Request request, InstanceMirror responseMirror) {
    final r = responseMirror.reflectee;
    if (r == null) {
      return new Response.internalServerError(body:
        'handler method returned null');
    }

    if (r is Response) {
      return r;
    }

    return doCreateResponse(request, responseMirror);
  }

  Response doCreateResponse(Request request, InstanceMirror responseMirror) {
    final body = createBody(request, responseMirror);
    final statusCode = createStatusCode(request, responseMirror, body);
    final headers = createHeaders(request, responseMirror, body);
    return new Response(statusCode, body: body, headers: headers);
  }

  createBody(Request request, InstanceMirror responseMirror);

  int createStatusCode(Request request, InstanceMirror responseMirror,
                       dynamic body) => successStatus;

  Map<String, String> createHeaders(Request request,
      InstanceMirror responseMirror, dynamic body) {
    final headers = {};
    if (setLocation) {
      final location = _getLocation(request, responseMirror);
      headers['Location'] = location;
    }
    addHeaders(headers, request, responseMirror, body);
    return headers;
  }

  String _getLocation(Request request, InstanceMirror responseMirror) {
    if (request.method != 'POST') {
      return request.requestedUri.toString();
    }

    final String id = getId(request, responseMirror);
    final requestPath = _url.fromUri(request.requestedUri);
    return _url.join(requestPath, id);
  }

  String getId(Request request, InstanceMirror responseMirror) {
    final id = responseMirror.getField(idField);
    return id.reflectee;
  }

  void addHeaders(Map<String, String> headers,
                  Request request, InstanceMirror responseMirror,
                  dynamic body) {}

  @override
  void _validate() {
    ensure(successStatus, isNotNull);
  }
}

class JsonResponseBinding extends BaseResponseBinding {

  const JsonResponseBinding(int successStatus, {bool setLocation: false,
    Symbol idField: #id
  })
      : super(successStatus, setLocation: setLocation, idField: idField);

  const JsonResponseBinding.ok()
      : super.ok();

  const JsonResponseBinding.created()
      : super.created();

  @override
  createBody(Request request, InstanceMirror responseMirror) =>
      JSON.encode(responseMirror.reflectee);

  @override
  void addHeaders(Map<String, String> headers,
                  Request request, InstanceMirror responseMirror,
                  dynamic body) {
      headers['Content-Type'] = 'application/json';
  }

}



typedef _BoundHandler(List<dynamic> positionalArguments, Request request,
    ResponseBinding responseBinding);
